# -*- coding: utf-8 -*-
from django.core.urlresolvers import reverse
from django.db import models
from django.dispatch import receiver
from django.utils.translation import ugettext_lazy as _
from djangocms_text_ckeditor.fields import HTMLField


class TermManager(models.Manager):
    def term_exists(self, name, exact=True):
        """
        Restituisce True se trova la voce di glossario `name`, False altrimenti.
        Se `exact` è True cerca una corrispondenza esatta, altrimenti si limita all'inizio del valore.
        """
        if name:
            if exact:
                return self.filter(name__iexact=name).exists()
            return self.filter(name__istartswith=name).exists()
        return False


class Term(models.Model):
    id = models.SlugField(max_length=50, primary_key=True, verbose_name=_("Chiave"))
    name = models.CharField(max_length=50, blank=False, null=False, verbose_name=_("Termine"))
    definition = HTMLField(blank=True, null=True, verbose_name=_("Definizione"))
    alias = models.ForeignKey('self', blank=True, null=True, verbose_name=_("Sinonimo di"))

    objects = TermManager()

    def get_url(self):
        if self.id:
            return reverse('glossario_term', kwargs={'current': self.id[0], 'pk': self.pk})

    def __unicode__(self):
        return u"%s" % self.name

    class Meta:
        verbose_name = _("termine")
        verbose_name_plural = _("termini")
        ordering = ('name',)


@receiver((models.signals.post_save, models.signals.post_delete), sender=Term)
def clear_index_cache(sender, **kwargs):
    if sender == Term:
        from .helpers import index_list

        index_list(reset=True)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import string
from django.core import cache

from .models import Term


def index_list(reset=False):
    """Restituisce l'elenco delle lettere e il relativo stato (active per le lettere che hanno voci sottostanti)"""
    cache_key = '{}:index_list'.format(__name__)
    index = None if reset else cache.cache.get(cache_key)

    if index is None:
        attive = {t.pk[0] for t in Term.objects.all()}
        index = [{'val': iniziale, 'active': iniziale.lower() in attive} for iniziale in string.uppercase]
        cache.cache.set(cache_key, index)

    return index




# -*- coding: utf-8 -*-
from __future__ import absolute_import
from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool
from django.utils.translation import ugettext_lazy as _


class GlossarioApp(CMSApp):
    name = _("Glossario")
    urls = ["glossario.urls"]


apphook_pool.register(GlossarioApp)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.conf.urls import patterns, url
from django.views.generic import RedirectView

from . import views

urlpatterns = patterns(
    '',
    url(r'^$', RedirectView.as_view(pattern_name='glossario_index', permanent=True), kwargs={'current': 'a'}, name='glossario'),
    url(r'^(?P<current>[a-z])/$', views.TermListView.as_view(), name='glossario_index'),
    # url(r'^(?P<current>[A-Z])/$', views.TermListView.as_view(), name='glossario_index'),
    url(r'^(?P<current>[a-z])/(?P<pk>[-a-z0-9]+)/$', views.TermDetailView.as_view(), name='glossario_term'),
    # url(r'^(?P<current>[a-zA-Z])/(?P<pk>[-a-zA-Z0-9]+)/$', views.TermDetailView.as_view(), name='glossario_term'),
)



# -*- coding: utf-8 -*-
from south.db import db
from south.v2 import SchemaMigration
from django.db import connection
try:
    import tinymce.models.HTMLField
except ImportError:
    definition_class = 'djangocms_text_ckeditor.fields.HTMLField'
else:
    definition_class = 'tinymce.models.HTMLField'


class Migration(SchemaMigration):

    def forwards(self, orm):
        table_names = connection.introspection.table_names()
        if 'glossario_term' not in table_names:
            # Adding model 'Term'
            db.create_table(u'glossario_term', (
                ('id', self.gf('django.db.models.fields.SlugField')(max_length=50, primary_key=True)),
                ('name', self.gf('django.db.models.fields.CharField')(max_length=50)),
                ('definition', self.gf(definition_class)(null=True, blank=True)),
                ('alias', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['glossario.Term'], null=True, blank=True)),
            ))
            db.send_create_signal(u'glossario', ['Term'])

    def backwards(self, orm):
        # Deleting model 'Term'
        db.delete_table(u'glossario_term')

    models = {
        u'glossario.term': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Term'},
            'alias': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['glossario.Term']", 'null': 'True', 'blank': 'True'}),
            'definition': (definition_class, [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.SlugField', [], {'max_length': '50', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['glossario']



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import logging
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext_lazy as _
from django.views.generic import ListView, DetailView

from . import models, settings
from .helpers import index_list

logger = logging.getLogger(__name__)


class GlossaryBaseView(object):
    model = models.Term

    def get_letter(self):
        return self.kwargs['current']

    def get_queryset(self):
        """Filtra in base alla lettera corrente"""
        queryset = super(GlossaryBaseView, self).get_queryset()
        initial = self.get_letter()
        if initial:
            queryset = queryset.filter(pk__istartswith=initial)
        return queryset.order_by('name')

    def get_title(self, context):
        return _("Glossario")

    def get_context_data(self, **kwargs):
        data = super(GlossaryBaseView, self).get_context_data(**kwargs)

        current = self.get_letter().upper()
        data.update(current=current, mainindex=index_list())
        data['page_title'] = self.get_title(data)
        if settings.TERMS_INDEX_COLUMNS and settings.TERMS_INDEX_COLUMNS > 1:
            items = self.get_queryset().count()
            if items:
                data['glossary_columns'] = settings.TERMS_INDEX_COLUMNS
                data['per_col'] = (items + settings.TERMS_INDEX_COLUMNS - 1) / settings.TERMS_INDEX_COLUMNS
        return data


class TermListView(GlossaryBaseView, ListView):
    def get_title(self, context):
        return _(u"Glossario: {current}").format(**context)


class TermDetailView(GlossaryBaseView, DetailView):
    def get_title(self, context):
        return _(u"Glossario: {term}").format(term=context['object'].name)

    def get_context_data(self, **kwargs):
        kwargs.setdefault('term_list', self.get_queryset())
        return super(TermDetailView, self).get_context_data(**kwargs)

    def render_to_response(self, context, **response_kwargs):
        term = context['object']
        if term.alias:
            seen = {term.pk}
            while term.alias:
                if term.alias.pk in seen:
                    logger.error("Glossary alias loop for {}".format(term))
                else:
                    seen.add(term.alias.pk)
                    term = term.alias
            return HttpResponseRedirect(term.get_url())
        return super(TermDetailView, self).render_to_response(context, **response_kwargs)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from django.contrib import admin

from . import models


class TermAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'alias', )
    # list_display_links = ('name', )
    search_fields = ('name', 'definition', )

admin.site.register(models.Term, TermAdmin)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Changing field 'Term.definition'
        db.alter_column(u'glossario_term', 'definition', self.gf('djangocms_text_ckeditor.fields.HTMLField')(null=True))

    def backwards(self, orm):
        try:
            import tinymce.models.HTMLField
        except ImportError:
            pass
        else:
            # Changing field 'Term.definition'
            db.alter_column(u'glossario_term', 'definition', self.gf('tinymce.models.HTMLField')(null=True))

    models = {
        u'glossario.term': {
            'Meta': {'ordering': "('name',)", 'object_name': 'Term'},
            'alias': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['glossario.Term']", 'null': 'True', 'blank': 'True'}),
            'definition': ('djangocms_text_ckeditor.fields.HTMLField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.SlugField', [], {'max_length': '50', 'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        }
    }

    complete_apps = ['glossario']


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

import glossario

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = glossario.__version__

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

if sys.argv[-1] == 'tag':
    print("Tagging the version on Bitbucket:")
    os.system("hg tag -m 'version %s' %s" % (version, version))
    os.system("hg push")
    sys.exit()


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

readme = read('README.rst')
history = read('HISTORY.rst').replace('.. :changelog:', '')

setup(
    name='django-glossa',
    version=version,
    description="""A glossary app for Django""",
    long_description=readme + '\n\n' + history,
    author='GS1 Italy',
    author_email='ict@gs1it.org',
    url='https://bitbucket.org/gs1it/django-glossa',
    packages=[
        'glossario',
    ],
    include_package_data=True,
    install_requires=[
    ],
    license="BSD",
    zip_safe=False,
    keywords='django-glossa',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Natural Language :: Italian',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3',
        # 'Programming Language :: Python :: 3.3',
        # 'Programming Language :: Python :: 3.4',
    ],
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import logging
from datetime import datetime
from django.contrib.admin.models import LogEntry
from django.contrib.contenttypes.models import ContentType
from django.contrib.sitemaps import Sitemap
from django.core.exceptions import ObjectDoesNotExist

from .models import Term

logger = logging.getLogger(__name__)


class GlossarioSitemap(Sitemap):
    MINDATE = datetime(2010, 2, 22)

    @classmethod
    def get_content_type(cls):
        return ContentType.objects.get_by_natural_key(app_label='glossario', model='term')

    def __init__(self, cutoff=None, *args, **kwargs):
        super(GlossarioSitemap, self).__init__()
        self.queryset = LogEntry.objects.filter(content_type=self.get_content_type())
        self.cutoff = cutoff

    def items(self):
        return Term.objects.filter(alias__isnull=True).order_by('name')

    def location(self, item):
        return item.get_url()

    def lastmod(self, item):
        try:
            return self.queryset.filter(object_id=item.pk).latest('action_time').action_time
        except ObjectDoesNotExist:
            pass
        except:
            logger.exception("{} lastmod error".format(self.__class__.__name__))
        return self.MINDATE

    priority = 0.6
    changefreq = 'weekly'
